# Licensed under a 3-clause BSD style license - see LICENSE.rst
from astropy.tests.helper import pytest
import numpy as np
import os
from ..spectrum import *
from ...tests.helpers import *
from astropy import units as u

class TestSpectrumManipulation_conversion:
  def test_wl2wn2wl(self):
    """
    Make sure that conversion from microns to wavenumbers and back works
    """
    testspec = generate_spectrum()
    assert testspec.x.unit == u.micron
    testspec.convert2wn()
    assert testspec.x.unit == u.kayser
    testspec.convert2wl()
    assert testspec.x.unit == u.micron
  def test_customconvert(self):
    """
    Make sure that custom unit conversion works as expected
    """
    testspec = generate_spectrum()
    assert testspec.x.unit == u.micron
    testspec.convert2(u.meter)
    assert testspec.x.unit == u.meter
    with pytest.raises(u.UnitsError):
        testspec.convert2(u.kg)

class TestSpectrumManipulation_convolution:
  def test_gaussianconvolution(self):
    """
    Test the functionality of the gaussian convolution
    """
    testspec = generate_cdespectrum()
    testspec.gconvolve(10.)
  def test_repeatedconvolution(self,recwarn):
    """
    Make sure that a warning is raised if convolution is repeated
    """
    testspec = generate_cdespectrum()
    testspec.gconvolve(10.)
    testspec.gconvolve(10.)
    w = recwarn.pop(RuntimeWarning)
    assert issubclass(w.category, RuntimeWarning)
  def test_gaussianpsfistoolarge(self):
    """
    Make sure that gaussian PSF can't be made too large
    """
    testspec = generate_cdespectrum()
    with pytest.raises(Exception):
      testspec.gpsf(len(testspec.x)+1)
  def test_smooth(self):
    """
    Test the various smoothing convolutions
    """
    testspec = generate_cdespectrum()
    testspec.smooth(window='flat')
    testspec = generate_cdespectrum()
    testspec.smooth(window='hanning')
    testspec = generate_cdespectrum()
    testspec.smooth(window='hamming')
    testspec = generate_cdespectrum()
    testspec.smooth(window='bartlett')
    testspec = generate_cdespectrum()
    testspec.smooth(window='blackman')
    testspec = generate_cdespectrum()
    testspec.smooth(window_len=2)
    testspec = generate_cdespectrum()
    with pytest.raises(ValueError):
      testspec.smooth(window='not a window')
    testspec = generate_cdespectrum()
    with pytest.raises(ValueError):
      testspec.smooth(window_len=1e10)

class TestSpectrumManipulation_misc:
  def test_interpolate(self):
    """
    Test interpolation between two different spectra
    """
    #normal function
    testspec1 = generate_cdespectrum()
    testspec2 = generate_absspectrum()
    testspec1.interpolate(testspec2)
    #trying to break it
    testspec1 = generate_cdespectrum()
    testspec2 = generate_absspectrum()
    testspec2.x = testspec2.x.value * u.kg
    with pytest.raises(Exception):
      testspec1.interpolate(testspec2)
    testspec1 = generate_cdespectrum()
    testspec2 = generate_absspectrum()
    testspec2.y = testspec2.y.value * u.kg
    with pytest.raises(Exception):
      testspec1.interpolate(testspec2)
  def test_subspectrum(self):
    """
    Test the extraction of a subspectrum from a spectrum
    """
    testspec = generate_cdespectrum()
    testspec.subspectrum(testspec.x[0].value+500,testspec.x[-1].value-500)
  def test_baseline_basic(self):
    """
    Test the baselining of a spectrum
    """
    testspec = generate_cdespectrum()
    testspec.baseline()
    testspec = generate_cdespectrum()
    testspec.baseline(windows=[[testspec.x[0].value+500,testspec.x[-1].value-500]],exclusive=True)
  def test_baseline_manual(self):
    """
    Test manual baseline functionality
    as far as the interactive mode allows non-interactively
    """
    testspec = generate_cdespectrum()
    cFig,cBaseliner = testspec.baseline(windows='manual')
    from matplotlib.backend_bases import MouseEvent, KeyEvent
    leftclick = MouseEvent('button_press_event',cFig.canvas,2000.,0.,button=1)
    midclick = MouseEvent('button_press_event',cFig.canvas,1000.,0.,button=2)
    rightclick = MouseEvent('button_press_event',cFig.canvas,1000.,0.,button=3)
    keypress_a = KeyEvent('key_press_event',cFig.canvas,'a')
    keypress_q = KeyEvent('key_press_event',cFig.canvas,'q')
    keypress_other = KeyEvent('key_press_event',cFig.canvas,'@')
    cBaseliner._Baseliner__mouse_press(leftclick)
    cBaseliner._Baseliner__mouse_press(midclick)
    cBaseliner._Baseliner__mouse_press(rightclick)
    cBaseliner._Baseliner__setlim(2000.)
    cBaseliner._Baseliner__setlim(2500.)
    cBaseliner._Baseliner__key_press(keypress_a)
    cBaseliner._Baseliner__key_press(keypress_other)
    cBaseliner._Baseliner__setlim(2800.)
    cBaseliner._Baseliner__setlim(2600.)
    cBaseliner._Baseliner__setlim(0.)


import errno
import os
import shutil

from distutils.core import Extension
from setuptools.command.build_ext import build_ext as SetuptoolsBuildExt

from ..utils import get_numpy_include_path, invalidate_caches
from ..version_helpers import get_pkg_version_module


def should_build_with_cython(package, release=None):
    """Returns the previously used Cython version (or 'unknown' if not
    previously built) if Cython should be used to build extension modules from
    pyx files.  If the ``release`` parameter is not specified an attempt is
    made to determine the release flag from `astropy.version`.
    """

    from ..setup_helpers import _module_state

    try:
        version_module = __import__(package + '.cython_version',
                                    fromlist=['release', 'cython_version'])
    except ImportError:
        version_module = None

    if release is None and version_module is not None:
        try:
            release = version_module.release
        except AttributeError:
            pass

    try:
        cython_version = version_module.cython_version
    except AttributeError:
        cython_version = 'unknown'

    # Only build with Cython if, of course, Cython is installed, we're in a
    # development version (i.e. not release) or the Cython-generated source
    # files haven't been created yet (cython_version == 'unknown'). The latter
    # case can happen even when release is True if checking out a release tag
    # from the repository
    if (_module_state['have_cython'] and
            (not release or cython_version == 'unknown')):
        return cython_version
    else:
        return False


# TODO: I think this can be reworked without having to create the class
# programmatically.
def generate_build_ext_command(packagename, release):
    """
    Creates a custom 'build_ext' command that allows for manipulating some of
    the C extension options at build time.  We use a function to build the
    class since the base class for build_ext may be different depending on
    certain build-time parameters (for example, we may use Cython's build_ext
    instead of the default version in distutils).

    Uses the default distutils.command.build_ext by default.
    """

    uses_cython = should_build_with_cython(packagename, release)

    if uses_cython:
        from Cython.Distutils import build_ext as basecls
    else:
        basecls = SetuptoolsBuildExt

    attrs = dict(basecls.__dict__)
    orig_run = getattr(basecls, 'run', None)
    orig_finalize = getattr(basecls, 'finalize_options', None)

    def finalize_options(self):
        # Add a copy of the _compiler.so module as well, but only if there are
        # in fact C modules to compile (otherwise there's no reason to include
        # a record of the compiler used)
        # Note, self.extensions may not be set yet, but
        # self.distribution.ext_modules is where any extension modules passed
        # to setup() can be found
        extensions = self.distribution.ext_modules
        if extensions:
            src_path = os.path.relpath(
                os.path.join(os.path.dirname(__file__), 'src'))
            shutil.copy2(os.path.join(src_path, 'compiler.c'),
                         os.path.join(self.package_name, '_compiler.c'))
            ext = Extension(self.package_name + '._compiler',
                            [os.path.join(self.package_name, '_compiler.c')])
            extensions.insert(0, ext)

        if orig_finalize is not None:
            orig_finalize(self)

        # Generate
        if self.uses_cython:
            try:
                from Cython import __version__ as cython_version
            except ImportError:
                # This shouldn't happen if we made it this far
                cython_version = None

            if (cython_version is not None and
                    cython_version != self.uses_cython):
                self.force_rebuild = True
                # Update the used cython version
                self.uses_cython = cython_version

        # Regardless of the value of the '--force' option, force a rebuild if
        # the debug flag changed from the last build
        if self.force_rebuild:
            self.force = True

    def run(self):
        # For extensions that require 'numpy' in their include dirs, replace
        # 'numpy' with the actual paths
        np_include = get_numpy_include_path()
        for extension in self.extensions:
            if 'numpy' in extension.include_dirs:
                idx = extension.include_dirs.index('numpy')
                extension.include_dirs.insert(idx, np_include)
                extension.include_dirs.remove('numpy')

            # Replace .pyx with C-equivalents, unless c files are missing
            for jdx, src in enumerate(extension.sources):
                if src.endswith('.pyx'):
                    pyxfn = src
                    cfn = src[:-4] + '.c'
                elif src.endswith('.c'):
                    pyxfn = src[:-2] + '.pyx'
                    cfn = src

                if not os.path.isfile(pyxfn):
                    continue

                if self.uses_cython:
                    extension.sources[jdx] = pyxfn
                else:
                    if os.path.isfile(cfn):
                        extension.sources[jdx] = cfn
                    else:
                        msg = (
                            'Could not find C file {0} for Cython file {1} '
                            'when building extension {2}. Cython must be '
                            'installed to build from a git checkout.'.format(
                                cfn, pyxfn, extension.name))
                        raise IOError(errno.ENOENT, msg, cfn)

        if orig_run is not None:
            # This should always be the case for a correctly implemented
            # distutils command.
            orig_run(self)

        # Update cython_version.py if building with Cython
        try:
            cython_version = get_pkg_version_module(
                    packagename, fromlist=['cython_version'])[0]
        except (AttributeError, ImportError):
            cython_version = 'unknown'
        if self.uses_cython and self.uses_cython != cython_version:
            package_dir = os.path.relpath(packagename)
            cython_py = os.path.join(package_dir, 'cython_version.py')
            with open(cython_py, 'w') as f:
                f.write('# Generated file; do not modify\n')
                f.write('cython_version = {0!r}\n'.format(self.uses_cython))

            if os.path.isdir(self.build_lib):
                # The build/lib directory may not exist if the build_py command
                # was not previously run, which may sometimes be the case
                self.copy_file(cython_py,
                               os.path.join(self.build_lib, cython_py),
                               preserve_mode=False)

            invalidate_caches()

    attrs['run'] = run
    attrs['finalize_options'] = finalize_options
    attrs['force_rebuild'] = False
    attrs['uses_cython'] = uses_cython
    attrs['package_name'] = packagename
    attrs['user_options'] = basecls.user_options[:]
    attrs['boolean_options'] = basecls.boolean_options[:]

    return type('build_ext', (basecls, object), attrs)



import numpy as np
import matplotlib.pyplot as plt
from astropy import units as u

class Baseliner:
  """
  A class for interactive baseliner of spectroscopic data.

  The class works by being fed a spectrum and a matplotlib axis on which
  it should be plotted. The spectrum is then plotted to the given axis,
  and a number of interactive options are made available to the user.

  Left-clicking with the mouse for the first time starts defining a window
  from the x-axis location of the click. A second click finishes the
  window between the locations of the first and second click.
  A third click will finish selecting windows, and perform the baselining.
  Alternately, right-clicking will cancel the last left-clicking action,
  allowing misplaced windows to be adjusted.

  Two keys are also accepted:
  Pressing "q" will cause the baselining process to be canceled,
  effectively skipping the baselining of this spectrum.
  Pressing "a" will allow an additional window to be defined, assuming
  one has been defined so far (by left-clicking twice to define its 
  boundaries).

  Attributes
  ----------
  windows : `list`
    A list of all the set windows.
  """
  def __init__(self,ax,spec):
    """
    Baseliner(ax,spec,order=1)

    Initialise the `Baseliner` class by giving it the target axis and
    spectrum.

    Parameters
    ----------
    ax : `matplotlib.axis`
      The matplotlib axis on which the interation will happen.
    spec : `omnifit.spectrum.BaseSpectrum`
      The spectrum which will be plotted as the visual reference on
      the given axis.
    """
    self.__ax = ax
    self.__spec = spec
    self.__x = spec.x.value
    self.__y = spec.y.value
    self.__limlo=None
    self.__limhi=None
    self.__minx=np.min(self.__x)
    self.__maxx=np.max(self.__x)
    self.__miny=np.min(self.__y)
    self.__maxy=np.max(self.__y)
    self.__ax.set_xlim(self.__minx,self.__maxx)
    self.__ax.set_ylim(self.__miny,self.__maxy)
    self.__specplot,=self.__ax.plot(self.__x,self.__y,'k-',drawstyle='steps-mid')
    self.__buttonListener = self.__ax.figure.canvas.mpl_connect('button_press_event', self.__mouse_press)
    self.__keyListener = self.__ax.figure.canvas.mpl_connect('key_press_event', self.__key_press)
    self.windows=[]
  def __key_press(self, event):
    if event.key=='q':
      self.__skip()
    if event.key=='a' and self.__limlo != None and self.__limhi != None:
      self.__addwindow(self.__limlo,self.__limhi)
      self.__ax.plot([self.__limlo,self.__limlo],[self.__miny,self.__maxy],'g-')
      self.__ax.plot([self.__limhi,self.__limhi],[self.__miny,self.__maxy],'g-')
      self.__remlim()
      self.__remlim()
      print 'Window added. Ready to receive another one.'
    else:
      return
  def __mouse_press(self, event):
    if event.button==1:
      self.__setlim(event.xdata)
    elif event.button==2:
      return
    elif event.button==3:
      self.__remlim()
  def __skip(self):
    plt.close()
  def __setlim(self,i_x):
    if self.__limlo==None:
      self.__limlo=i_x
      self.__limloplot,=self.__ax.plot([i_x,i_x],[self.__miny,self.__maxy],'b-')
      self.__ax.figure.canvas.draw()
    elif self.__limhi==None:
      self.__limhi=i_x
      self.__limhiplot,=self.__ax.plot([i_x,i_x],[self.__miny,self.__maxy],'b-')
      self.__ax.figure.canvas.draw()
      print 'Ready for finalising. Press once more to do so, or press a to add another window.'
    else:
      self.__finalise()
  def __remlim(self):
    if self.__limhi!=None:
      self.__limhi=None
      self.__limhiplot.set_ydata([self.__miny,self.__miny])
      self.__ax.figure.canvas.draw()
    elif self.__limlo!=None:
      self.__limlo=None
      self.__limloplot.set_ydata([self.__miny,self.__miny])
      self.__ax.figure.canvas.draw()
    else:
      print 'No limits to cancel.'
  def __addwindow(self,limlo,limhi):
    if limhi < limlo:
      limlo,limhi = limhi,limlo
    self.windows.append([limlo,limhi])
  def __finalise(self):
    self.__addwindow(self.__limlo,self.__limhi)
    self.__ax.figure.canvas.mpl_disconnect(self.__buttonListener)
    self.__ax.figure.canvas.mpl_disconnect(self.__keyListener)
    plt.close(self.__ax.figure)

#Units definitions
unit_od = u.def_unit('optical depth',doc='Optical depth of radiation')
unit_opticaldepth = unit_od

#--------------------------
#Misc. function definitions
#--------------------------
def cde_correct(wn,n,k):
  """
  cde_correct(wn,n,k)

  Generate a CDE-corrected spectrum from a complex refractive index
  spectrum.

  Parameters
  ----------
  wn : `numpy.ndarray`
    The frequency data of the input spectrum, in reciprocal
    wavenumbers (cm^-1).
  n : `numpy.ndarray`
    The real component of the complex refractive index spectrum.
  k : `numpy.ndarray`
    The imaginary component of the complex refractive index spectrum.

  Returns
  -------
  A list containing the following numpy arrays, in given order:
    * The spectrum of the absorption cross section of the simulated grain.
    * The spectrum of the absorption cross section of the simulated grain,
      normalized by the volume distribution of the grain. This parameter
      is the equivalent of optical depth in most cases.
    * The spectrum of the scattering cross section of the simulated grain,
      normalized by the volume distribution of the grain.
    * The spectrum of the total cross section of the simulated grain.    
  """
  wl=1.e4/wn
  m=np.vectorize(complex)(n,k)
  cabs_vol=np.empty(0)
  cabs=np.empty(0)
  cscat_vol=np.empty(0)
  for c_wn,c_wl,c_m in zip(wn,wl,m):
    m2=c_m**2.0
    im_part=((m2/(m2-1))*np.log(m2)).imag
    cabs_vol=np.hstack([cabs_vol,2.*(2.*np.pi/c_wl)*im_part])
    t_cabs=c_wn*(2.0*c_m.imag/(c_m.imag-1))*np.log10(c_m.imag)
    cabs=np.hstack([cabs,t_cabs])
    cscat_vol=np.hstack([cscat_vol,(c_wn**3.0/(6*np.pi))*t_cabs])
  ctot=cabs+cscat_vol
  return cabs,cabs_vol,cscat_vol,ctot


import inspect
import os
import pkgutil
import re
import shutil
import subprocess
import sys
import textwrap

from distutils import log
from distutils.cmd import DistutilsOptionError

import sphinx
from sphinx.setup_command import BuildDoc as SphinxBuildDoc

from ..utils import minversion


PY3 = sys.version_info[0] >= 3


class AstropyBuildSphinx(SphinxBuildDoc):
    """
    A version of the ``build_sphinx`` command that uses the version of Astropy
    that is built by the setup ``build`` command, rather than whatever is
    installed on the system.  To build docs against the installed version, run
    ``make html`` in the ``astropy/docs`` directory.

    This also automatically creates the docs/_static directories--this is
    needed because GitHub won't create the _static dir because it has no
    tracked files.
    """

    description = 'Build Sphinx documentation for Astropy environment'
    user_options = SphinxBuildDoc.user_options[:]
    user_options.append(('warnings-returncode', 'w',
            'Parses the sphinx output and sets the return code to 1 if there '
            'are any warnings. Note that this will cause the sphinx log to '
            'only update when it completes, rather than continuously as is '
            'normally the case.'))
    user_options.append(('clean-docs', 'l',
            'Completely clean previous builds, including '
            'automodapi-generated files before building new ones'))
    user_options.append(('no-intersphinx', 'n',
            'Skip intersphinx, even if conf.py says to use it'))
    user_options.append(('open-docs-in-browser', 'o',
            'Open the docs in a browser (using the webbrowser module) if the '
            'build finishes successfully.'))

    boolean_options = SphinxBuildDoc.boolean_options[:]
    boolean_options.append('warnings-returncode')
    boolean_options.append('clean-docs')
    boolean_options.append('no-intersphinx')
    boolean_options.append('open-docs-in-browser')

    _self_iden_rex = re.compile(r"self\.([^\d\W][\w]+)", re.UNICODE)

    def initialize_options(self):
        SphinxBuildDoc.initialize_options(self)
        self.clean_docs = False
        self.no_intersphinx = False
        self.open_docs_in_browser = False
        self.warnings_returncode = False

    def finalize_options(self):
        #Clear out previous sphinx builds, if requested
        if self.clean_docs:
            dirstorm = [os.path.join(self.source_dir, 'api')]
            if self.build_dir is None:
                dirstorm.append('docs/_build')
            else:
                dirstorm.append(self.build_dir)

            for d in dirstorm:
                if os.path.isdir(d):
                    log.info('Cleaning directory ' + d)
                    shutil.rmtree(d)
                else:
                    log.info('Not cleaning directory ' + d + ' because '
                             'not present or not a directory')

        SphinxBuildDoc.finalize_options(self)

    def run(self):
        # TODO: Break this method up into a few more subroutines and
        # document them better
        import webbrowser

        if PY3:
            from urllib.request import pathname2url
        else:
            from urllib import pathname2url

        # This is used at the very end of `run` to decide if sys.exit should
        # be called. If it's None, it won't be.
        retcode = None

        # If possible, create the _static dir
        if self.build_dir is not None:
            # the _static dir should be in the same place as the _build dir
            # for Astropy
            basedir, subdir = os.path.split(self.build_dir)
            if subdir == '':  # the path has a trailing /...
                basedir, subdir = os.path.split(basedir)
            staticdir = os.path.join(basedir, '_static')
            if os.path.isfile(staticdir):
                raise DistutilsOptionError(
                    'Attempted to build_sphinx in a location where' +
                    staticdir + 'is a file.  Must be a directory.')
            self.mkpath(staticdir)

        # Now make sure Astropy is built and determine where it was built
        build_cmd = self.reinitialize_command('build')
        build_cmd.inplace = 0
        self.run_command('build')
        build_cmd = self.get_finalized_command('build')
        build_cmd_path = os.path.abspath(build_cmd.build_lib)

        ah_importer = pkgutil.get_importer('astropy_helpers')
        ah_path = os.path.abspath(ah_importer.path)

        # Now generate the source for and spawn a new process that runs the
        # command.  This is needed to get the correct imports for the built
        # version
        runlines, runlineno = inspect.getsourcelines(SphinxBuildDoc.run)
        subproccode = textwrap.dedent("""
            from sphinx.setup_command import *

            os.chdir({srcdir!r})
            sys.path.insert(0, {build_cmd_path!r})
            sys.path.insert(0, {ah_path!r})

        """).format(build_cmd_path=build_cmd_path, ah_path=ah_path,
                    srcdir=self.source_dir)
        # runlines[1:] removes 'def run(self)' on the first line
        subproccode += textwrap.dedent(''.join(runlines[1:]))

        # All "self.foo" in the subprocess code needs to be replaced by the
        # values taken from the current self in *this* process
        subproccode = self._self_iden_rex.split(subproccode)
        for i in range(1, len(subproccode), 2):
            iden = subproccode[i]
            val = getattr(self, iden)
            if iden.endswith('_dir'):
                # Directories should be absolute, because the `chdir` call
                # in the new process moves to a different directory
                subproccode[i] = repr(os.path.abspath(val))
            else:
                subproccode[i] = repr(val)
        subproccode = ''.join(subproccode)

        # This is a quick gross hack, but it ensures that the code grabbed from
        # SphinxBuildDoc.run will work in Python 2 if it uses the print
        # function
        if minversion(sphinx, '1.3'):
            subproccode = 'from __future__ import print_function' + subproccode

        if self.no_intersphinx:
            # the confoverrides variable in sphinx.setup_command.BuildDoc can
            # be used to override the conf.py ... but this could well break
            # if future versions of sphinx change the internals of BuildDoc,
            # so remain vigilant!
            subproccode = subproccode.replace('confoverrides = {}',
                'confoverrides = {\'intersphinx_mapping\':{}}')

        log.debug('Starting subprocess of {0} with python code:\n{1}\n'
                  '[CODE END])'.format(sys.executable, subproccode))

        # To return the number of warnings, we need to capture stdout. This
        # prevents a continuous updating at the terminal, but there's no
        # apparent way around this.
        if self.warnings_returncode:
            proc = subprocess.Popen([sys.executable],
                                    stdin=subprocess.PIPE,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.STDOUT)
            stdo, stde = proc.communicate(subproccode.encode('utf-8'))

            print(stdo)

            stdolines = stdo.splitlines()

            if b'build succeeded.' in stdolines:
                retcode = 0
            else:
                retcode = 1

            if retcode != 0:
                if os.environ.get('TRAVIS', None) == 'true':
                    #this means we are in the travis build, so customize
                    #the message appropriately.
                    msg = ('The build_sphinx travis build FAILED '
                           'because sphinx issued documentation '
                           'warnings (scroll up to see the warnings).')
                else:  # standard failure message
                    msg = ('build_sphinx returning a non-zero exit '
                           'code because sphinx issued documentation '
                           'warnings.')
                log.warn(msg)

        else:
            proc = subprocess.Popen([sys.executable], stdin=subprocess.PIPE)
            proc.communicate(subproccode.encode('utf-8'))

        if proc.returncode == 0:
            if self.open_docs_in_browser:
                if self.builder == 'html':
                    absdir = os.path.abspath(self.builder_target_dir)
                    index_path = os.path.join(absdir, 'index.html')
                    fileurl = 'file://' + pathname2url(index_path)
                    webbrowser.open(fileurl)
                else:
                    log.warn('open-docs-in-browser option was given, but '
                             'the builder is not html! Ignoring.')
        else:
            log.warn('Sphinx Documentation subprocess failed with return '
                     'code ' + str(proc.returncode))

        if retcode is not None:
            # this is potentially dangerous in that there might be something
            # after the call to `setup` in `setup.py`, and exiting here will
            # prevent that from running.  But there's no other apparent way
            # to signal what the return code should be.
            sys.exit(retcode)



# Licensed under a 3-clause BSD style license - see LICENSE.rst
from astropy.tests.helper import pytest
import numpy as np
import os
from ..spectrum import *
from ...tests.helpers import *

class TestSpectrumInfo:
  def test_minmax(self):
    """
    Make sure that min and max work
    """
    testspec = generate_cdespectrum()
    testspec.min()
    testspec.min(checkrange=[2000.,2050.])
    testspec.max()
    testspec.max(checkrange=[2000.,2050.])
  def test_info(self):
    """
    Make sure that spectrum info is produced
    """
    testspec = generate_cdespectrum()
    testspec.info()


from __future__ import absolute_import

from subprocess import *


def check_output(*popenargs, **kwargs):
    r"""Run command with arguments and return its output as a byte
    string.

    If the exit code was non-zero it raises a CalledProcessError.  The
    CalledProcessError object will have the return code in the
    returncode
    attribute and output in the output attribute.

    The arguments are the same as for the Popen constructor.  Example::

    >>> check_output(["ls", "-l", "/dev/null"])
    'crw-rw-rw- 1 root root 1, 3 Oct 18  2007 /dev/null\n'

    The stdout argument is not allowed as it is used internally.
    To capture standard error in the result, use stderr=STDOUT.::

    >>> check_output(["/bin/sh", "-c",
    ...               "ls -l non_existent_file ; exit 0"],
    ...              stderr=STDOUT)
    'ls: non_existent_file: No such file or directory\n'
    """
    if 'stdout' in kwargs:
        raise ValueError('stdout argument not allowed, it will be overridden.')
    process = Popen(stdout=PIPE, *popenargs, **kwargs)
    output, unused_err = process.communicate()
    retcode = process.poll()
    if retcode:
        cmd = kwargs.get("args")
        if cmd is None:
            cmd = popenargs[0]
        raise CalledProcessError(retcode, cmd)
    return output



import numpy as np
from lmfit import minimize, Parameters, Parameter
from astropy import units as u
from .. import spectrum
from functions import *

class Fitter():
  """
  A class for multi-component fitting to spectroscopic data of ices.

  This is the heart of Omnifit, which receives spectra from the spectrum
  module, and is then capable of fitting an arbitrary number of different
  components to the target spectrum thus designated.

  Attributes
  ----------
  target_x : `numpy.ndarray`
    The x axis of the target spectrum, e.g. the wavelength.
  target_y : `numpy.ndarray`
    The y axis of the target spectrum, e.g. the optical depth.
  target_dy : `float`
    A single number expressing the average uncertainty of the y
    axis data.
  modelname : `string`
    A human-readable name for the model being fitted.
  psf : `Nonetype`, `numpy.ndarray`, or `astropy.convolution.Kernel`
    If set, this attribute can be used to give a kernel which should
    be used to convolve all the fitted data with.
  fitrange : `Nonetype` or `list`
    If set, this specifies the inclusive limits to which
    the fitting should be performed in x axis coordinates.
    For example a fitrange of [[200,250],[300,350]] sets
    two fitting windows of 200 to 250, and 300 to 350.
  color : `string`
    A string inidcating the desired plotting color of the target
    data, in a format understandable by matplotlib.
  funclist : `list`
    A list containing all the fittable functions. Each list entry
    is a dictionary containing the following keys and values:

      * 'name' : A human-readable name for the function being fitted,
        in string format.
      * 'color' : A string inidcating the desired plotting color of
        the data, in a format understandable by matplotlib.
      * 'type' : A string indicating what type of data the function
        consists of. It can be either 'analytical' or 'empirical',
        indicating an analytical function or empirical spectrum,
        respectively.
      * 'shape' : The shape of the function being fitted. In the case
        of an analytical function, this is a string indicating the
        callable name of the function. In the case of an empirical
        spectrum, this is the y-axis data from the spectrum.
      * 'params' : an lmfit `Parameters` instance containing the fitting
        parameters appropriate to the data being fitted.

  fitpars : `Parameters`
    This is where the fitting parameters are stored during and after
    minimization.
  fitres : `Minimizer`
    The fitting results are stored in this class, as documented in
    lmfit.
  """
  def __init__(self,x,y,dy=1.0,modelname='Unknown model',psf=None,fitrange=None,color='black'):
    """
    Fitter(x,y,dy=1.0,modelname='Unknown model',psf=None,fitrange=None,color='black')

    Constructor for the Fitter class. Initialisation happens by
    designating the target spectrum.

    Parameters
    ----------
    x : `numpy.ndarray`
      The x axis of the target spectrum, e.g. the wavelength.
    y : `numpy.ndarray`
      The y axis of the target spectrum, e.g. the optical depth.
    dy : `float`, optional
      A single number expressing the average uncertainty of the y
      axis data.
    modelname : `string`, optional
      A human-readable name for the model being fitted.
    psf : Nonetype or numpy.ndarray or astropy.convolution.Kernel, optional
      This attribute can be used to give a kernel which should be
      used to convolve all the fitted data with.
    fitrange : `Nonetype` or `list`, optional
      If set, this specifies the inclusive limits to which
      the fitting should be performed in x axis coordinates.
      For example a fitrange of [[200,250],[300,350]] sets
      two fitting windows of 200 to 250, and 300 to 350.
    color : `string`, optional
      A string inidcating the desired plotting color of the target
      data, in a format understandable by matplotlib.
    """
    if len(x) != len(y):
      raise RuntimeError('Input arrays have different sizes.')
    self.target_x=x
    self.target_y=y
    self.target_dy=dy
    self.modelname=modelname
    self.psf=psf
    self.fitrange=fitrange
    self.color=color
    self.funclist=[]
  @classmethod
  def fromspectrum(cls,spectrum,**kwargs):
    """
    Fitter.fromspectrum(spectrum,**kwargs)

    An alternate way to initialise Fitter, by directly giving it
    a spectrum. Extracted data from the spectrum are the x, y,
    and (if the spectrum has been baselined) dy parameters.

    Parameters
    ----------
    spectrum : `omnifit.spectrum.BaseSpectrum` or its child class
      The input spectrum.
    **kwargs : Arguments, optional
      Additional initialisation arguments can be passed to `Fitter`
      using this. Note that x and y (and dy, if applicable) are defined
      using the data contained in the input spectrum.
    """
    if spectrum.baselined:
      return cls(spectrum.x.value,spectrum.y.value,spectrum.dy,**kwargs)
    else:
      return cls(spectrum.x.value,spectrum.y.value,**kwargs)
  def add_empirical(self,spectrum,params,funcname=None,color='red'):
    """
    add_empirical(spectrum,params,funcname=None,color='red')

    Add empirical data in the form of a spectrum to the fitting list.
    The spectrum must be interpolated to match the target x axis.

    Parameters
    ----------
    spectrum : `spectrum.BaseSpectrum`
      The input spectrum.
    params : `Parameters`
      The input parameters. Specifically this must contain
      the 'mul' parameter, which indicates what value the
      spectrum will be multiplied with during fitting.
    funcname : `Nonetype` or `string`, optional
      A human-readable name for the data being fitted.
      If this is left as None, the name of the spectrum will
      be used.
    color : `string`, optional
      A string inidcating the desired plotting color of the
      data, in a format understandable by matplotlib.
    """
    if not(funcname):
      funcname=spectrum.name
    if not np.all(spectrum.x.value == self.target_x):
      raise RuntimeError('Input spectrum x axis does not match the target spectrum x axis.')
    self.funclist.append({'type':'empirical','shape':spectrum.y.value,'params':params,'name':funcname,'color':color})
  def add_analytical(self,shape,params,funcname='Unknown function',color='red'):
    """
    add_analytical(shape,params,funcname=None,color='red')

    Add analytical data in the form of a callable function to the
    fitting list.

    Parameters
    ----------
    shape : `string`
      The callable name of the function to be fitted.
    params : `Parameters`
      The input parameters. These should be formatted in a way that
      the function defined by shape can understand them, and that
      function should be created in such a way that it can make use
      of lmfit parameters.
    funcname : `string`, optional
      A human-readable name for the data being fitted.
    color : `string`, optional
      A string inidcating the desired plotting color of the
      data, in a format understandable by matplotlib.
    """
    self.funclist.append({'type':'analytical','shape':shape,'params':params,'name':funcname,'color':color})

  def perform_fit(self,**kwargs):
    """
    perform_fit(**kwargs)

    Uses `minimize` in lmfit to perform least-squares fitting of all the
    functions in the function list to the target data.

    Parameters
    ----------
    **kwargs : Arguments, optional
      This can be used to give additional arguments for `minimize`.
    """
    self.fitpars = self.__extract_pars()
    self.fitres=minimize(self.__fit_residual,self.fitpars,**kwargs)
    if not(self.fitres.success):
      raise RuntimeError('Fitting failed!')
  def __fit_residual(self,params,custrange=None):
    """
    __fit_residual(params,custrange=None)

    This is an internal function used for calculating the total
    residual of the data against the fittings function(s), given
    a set of lmfit parameters. The residual calculation can also
    be limited to a specific x axis range.

    Parameters
    ----------
    params : `Parameters`
      The parameters used for calculating the residual.
    custrange : `Nonetype` or `list`, optional
      If set, this specifies the inclusive range within which
      the residual is calculated. Otherwise the fitting range
      specified during Initialisation is used.

    Returns
    -------
    The residual function within the fitting range with the given
    lmfit parameters.
    """
    if custrange==None:
      fitrange=self.fitrange
    else:
      fitrange=custrange
    residual=1.0*self.target_y
    totModel=np.zeros(len(residual))
    for indFunc,cFunc in enumerate(self.funclist):
      oPar=Parameters()
      cParlist = cFunc['params']
      for cPar in cParlist.values():
        cParams=params[self.__func_ident(indFunc)+cPar.name]
        oPar.add(cPar.name,
                 value=cParams.value,vary=cParams.vary,
                 min=cParams.min,max=cParams.max,
                 expr=cParams.expr)
      residual-=self.__parse_function(oPar,cFunc)
    #Crop out not-numbers and fitting range exterior if necessary
    if np.any(fitrange):
      fitInd=np.isinf(residual)
      for cRange in fitrange:
        fitInd=np.logical_or(fitInd,np.logical_and(
               np.less_equal(cRange[0],self.target_x),
               np.greater_equal(cRange[1],self.target_x)))
    else:
      fitInd=np.isfinite(residual)
    return residual[fitInd]
  def chisq(self,checkrange=None):
    """ 
    chisq(checkrange=None)

    Return chi squared of fit, either in a custom range 
    or in the range used by the fit.

    Parameters
    ----------
    checkrange : `Nonetype` or `list`, optional
      If set, this specifies the inclusive range within which
      the chi squared value is calculated. Otherwise the fitting 
      range specified during Initialisation is used.

    Returns
    -------
    The chi squared within the desired ranged.
    """
    residual = self.__fit_residual(self.fitpars,custrange=checkrange)
    return np.sum((residual**2.0)/(self.target_dy**2.0))
  def plot_fitresults(self,ax,lw=[1,2,3],color_total='blue',legend=True,**kwargs):
    """
    plot_fitresults(ax,lw=[1,2,3],color_total='blue',legend=True,**kwargs)
    
    Plot the fitting results to the given matplotlib axis, with a
    number of optional parameters specifying how the different plottable
    components are presented.

    Parameters
    ----------
    axis : `matplotlib.axis`
      The axis which the plot will be generated in.
    lw : `list`, optional
      This list of 3 numbers specifies the line widths of the target
      spectrum, the fitted functions, and the total fit, respectively.
    color_total : `string`, optional
      A string inidcating the desired plotting color of the total sum
      of the fit results, in a format understandable by matplotlib.
      The colors of the target spectrum and the fitted functions are
      specified during their initialisation and addition.
    legend : `bool`, optional
      If set to True, a legend is automatically created using the
      target spectrum and fitted function names.
    **kwargs : Arguments, optional
      This can be used to pass additional arguments
      to `matplotlib.pyplot.plot`, which is used by this 
      method for its plotting.
    """
    ax.plot(self.target_x,self.target_y,color=self.color,lw=lw[0],**kwargs)
    legList = [self.modelname]
    #totres=self.targ_y+self.fitres.residual
    totRes=np.zeros(len(self.target_y))
    for indFunc,cFunc in enumerate(self.funclist):
      oPar=Parameters()
      cParList = cFunc['params']
      cCol = cFunc['color']
      for cPar in cParList.values():
        cFitPar=self.fitpars[self.__func_ident(indFunc)+cPar.name]
        oPar.add(cPar.name,
                 value=cFitPar.value,vary=cFitPar.vary,
                 min=cFitPar.min,max=cFitPar.max,
                 expr=cFitPar.expr)
      funcRes = self.__parse_function(oPar,cFunc)
      totRes+=funcRes
      ax.plot(self.target_x,funcRes,lw=lw[1],color=cCol,**kwargs)
      legList.append(cFunc['name'])
    legList.append('Total fit')
    ax.plot(self.target_x,totRes,lw=lw[2],color=color_total,**kwargs)
    if legend:
      ax.legend(legList,shadow=True)
  def fitresults_tofile(self,filename,detection_threshold=5.0):
    """
    fitresults_tofile(filename)

    Export fit results to two output files which are intended to be
    easily readable and paraseable with other software.

    The first file is filename.csv, which contains x and y data of
    the fitted models, as would be visualized in a plotted fit result.
    The first column of the csv is the x value, which is shared by all
    models.
    The second column is the y value of data that was being fitted to.
    The third column is total sum of fitted models.
    The fourth to Nth columns are the individual models, in the order
    described in the second file, filename.xml.

    The second file, filename.xml is an XML file containing additional
    information about the fitted data and the fit results which are not
    easily representable in a csv-formatted file. This data is
    formatted using the following XML elements:

      * INFO : Contains all the other elements described below, and has
        the attribute "file", which is the name of the csv file pair of
        this xml file.
      * MODELNAME : Contains the name of the model.
      * HAVEPSF : A boolean value indicating whether there is a PSF
        associated with the model.
      * RMS_DATA : The uncertainty of the data.
      * NUMBER_FUNCTIONS : An integer indicating how many functions
        have been fitted to the total data.

    In addition to the above elements, each fitted function has its own
    element, designated FUNCTION, having the attribute "name" which is
    the name of the function. FUNCTION contains the following elements:

      * TYPE : If the function is an empirical one, this contains the
        string "empirical". Otherwise it contains the name of the
        called analytical function.
      * DETECTION : When generating the contents of this element,
        The method is_nondet with the detection threshold designated
        by the parameter detection_threshold. The result given by
        the method is indicated here with a "True" or "False"
        depending on whether the result is considered a detection.
      * CSV_COLUMN : Indicates which column in the CSV contains the
        fitted data for this function.
      * NUMBER_PARAMS : Inidicates how many parameters are used by
        this function i.e. the number of PARAMETER elements.

    Finally, contained within each FUNCTION element is a number of
    PARAMETER elements, which list the best-fit data for each fitted
    parameter pertaining to that function. Each PARAMETER element
    contains the attribute "name", which tells the name of the
    parameter. In addition the following elements are contained by
    each PARAMETER element:

      * VALUE : The best-fit value for this parameter.

    Parameters
    ----------
    filename : `string`
      The extensionless version of the desired filename which the
      data should be exported to. As a result the files
      "filename.csv" and "filename.xml" are created.
    detection_threshold : `float`, optional
      The threshold of detection to be used in determining whether
      the value contained by the DETECTION element is true or not.
    """
    filename_csv = filename+'.csv'
    filename_xml = filename+'.xml'
    file_xml = open(filename_xml,'w')
    file_xml.write('<!-- Automatically generated information file for csv file '+filename_csv+'-->\n')
    file_xml.write('<INFO file="'+filename_csv+'">\n')
    file_xml.write('<MODELNAME>'+self.modelname+'</MODELNAME>\n')
    file_xml.write('<HAVEPSF>'+str(self.psf != None)+'</HAVEPSF>\n')
    file_xml.write('<RMS_DATA>'+str(self.target_dy)+'</RMS_DATA>\n')
    file_xml.write('<NUMBER_FUNCTIONS>'+str(len(self.funclist))+'</NUMBER_FUNCTIONS>\n')
    outdata_csv = np.vstack([self.target_x,self.target_y])
    outdata_functions = np.empty([0,len(self.target_x)])
    totRes = np.zeros(len(self.target_x))
    for indFunc,cFunc in enumerate(self.funclist):
      file_xml.write('<FUNCTION name="'+cFunc['name']+'">\n')
      file_xml.write('<TYPE>')
      if cFunc['type'] == 'analytical':
        file_xml.write(cFunc['shape'])
      elif cFunc['type'] == 'empirical':
        file_xml.write('empirical')
      else:
        file_xml.write('unknown'+'\n')
      file_xml.write('</TYPE>\n')
      file_xml.write('<DETECTION>'+str(not self.is_nondet(sigma=detection_threshold)[cFunc['name']])+'</DETECTION>\n')
      file_xml.write('<CSV_COLUMN>'+str(indFunc+3)+'</CSV_COLUMN>\n')
      cParlist = cFunc['params']
      file_xml.write('<NUMBER_PARAMS>'+str(len(cParlist))+'</NUMBER_PARAMS>\n')
      oPar=Parameters()
      for cPar in cParlist.values():
        file_xml.write('<PARAMETER name="'+cPar.name+'">\n')
        cFitPar=self.fitpars[self.__func_ident(indFunc)+cPar.name]
        oPar.add(cPar.name,
                 value=cFitPar.value,vary=cFitPar.vary,
                 min=cFitPar.min,max=cFitPar.max,
                 expr=cFitPar.expr)
        file_xml.write('<VALUE>'+str(cFitPar.value)+'</VALUE>\n')
        file_xml.write('</PARAMETER>\n')
      funcRes = self.__parse_function(oPar,cFunc)
      outdata_functions = np.vstack([outdata_functions,funcRes])
      totRes += funcRes
      file_xml.write('</FUNCTION>\n')
    file_xml.write('</INFO>')
    file_xml.close()
    outdata_csv = np.vstack([outdata_csv,totRes,outdata_functions])
    np.savetxt(filename_csv,outdata_csv.transpose(),delimiter=',',header='For info, see '+filename_xml)
  def is_nondet(self,sigma=5.0):
    """
    is_nondet(sigma=5.0)

    Determines whether the fitted functions in the function list can
    be considered detections or non-detections using the given detection
    thereshold. This is done by comparing the peak of the fitted function
    within the fitting range to a multiple (set by the parameter sigma)
    of the RMS noise in the target data.
    It should be emphasized that unless the dy attribute has been set
    during the fitter class initialisation, the results returned by this
    method are meaningless.

    Parameters
    ----------
    sigma : `float`, optional
      The multiplier that should be applied to the noise when comparing
      it against the fitted function peaks.

    Returns
    -------
    A dictionary containing boolean values for each function (with
    their names as the keys) and the total fit (key 'total'), with
    True indicating that the function is considered a non-detection
    using the criteria outlined above.
    """
    minY = sigma*self.target_dy
    print self.target_dy
    out = {}
    totRes = np.zeros(len(self.target_x))
    for indFunc,cFunc in enumerate(self.funclist):
      cParlist = cFunc['params']
      oPar=Parameters()
      for cPar in cParlist.values():
        cFitPar=self.fitpars[self.__func_ident(indFunc)+cPar.name]
        oPar.add(cPar.name,
                 value=cFitPar.value,vary=cFitPar.vary,
                 min=cFitPar.min,max=cFitPar.max,
                 expr=cFitPar.expr)
      funcRes = self.__parse_function(oPar,cFunc)
      if np.max(funcRes) < minY:
        out[cFunc['name']] = True
      else:
        out[cFunc['name']] = False
      totRes += funcRes
    if np.max(totRes) < minY:
      out['total'] = True
    else:
      out['total'] = False
    return out
  def fit_results(self):
    """
    fit_results()

    Return the fitting results as a dictionary.

    Parameters
    ----------
    None

    Returns
    -------
    A dictionary containing all the individual functions which were
    fitted. The key-value combinations of this dictionary consist of
    the function name, and its lmfit Parameters instance, which
    contains the best-fit results.
    """
    oResults={}
    for indFunc,cFunc in enumerate(self.funclist):
      oKeyname_base=cFunc['name']
      oKeyind=0
      oKeyname=oKeyname_base
      while oResults.__contains__(oKeyname): #In case of duplicate function names
        oKeyind+=1
        oKeyname=oKeyname_base+'(duplicate '+str(oKeyind)+')'
      oResults[cFunc['name']]=self.__fit_result(indFunc)
    return oResults
  def __fit_result(self,index):
    """
    __fit_result(index)

    Return fitting results for a specific function in the internal
    function list.

    Parameters
    ----------
    index : `int`
      Desired index of the function to fetch from the function lsit.

    Returns
    -------
    An `Parameters` instance containing the fitting
    results for the desired function.
    """
    oParlist=self.funclist[index]['params']
    for cParname in oParlist.keys():
      coPar=self.fitpars[self.__func_ident(index)+cParname]
      coPar.name=cParname
      oParlist[cParname]=coPar
    return oParlist
  def __parse_function(self,params,function):
    """
    __parse_function(params,function)

    Parse the input function, insert parameters, return result.

    Parameters
    ----------
    params : `Parameters`
      The lmfit `Parameters` instance to use as input parameters.
    function : `dict`
      A dictionary formatted in the style that the entries inside
      funclist are formatted

    Returns
    -------
    The result of the given function with given parameters.
    """
    if function['type']=='empirical':
      funcres=muldata(function['shape'],params['mul'].value)
    elif function['type']=='analytical':
      funcres=globals()[function['shape']](self.target_x,params,self.psf)
    else:
      raise RuntimeError('Unknown function type!')
    return funcres
  def __extract_pars(self):
    """
    __extract_pars()

    Extracts the paramers from the function list and converts them to
    a single lmfit Parameters instance, which can then be manipulated
    by the residual minimization routines.

    Parameters
    ----------
    None

    Returns
    -------
    An lmfit `Parameters` instance containing the parameters
    of *all* the fittable functions in a single place.
    """
    oPars=Parameters()
    for indFunc,cFunc in enumerate(self.funclist):
      cParlist = cFunc['params']
      for cPar in cParlist.values():
        oPars.add(self.__func_ident(indFunc)+cPar.name,
                  value=cPar.value,vary=cPar.vary,
                  min=cPar.min,max=cPar.max,
                  expr=cPar.expr)
    return oPars
  def __func_ident(self,index):
    """ 
    __func_ident(index)

    Generate a unique prefix string for a function, which can be
    used by `__extract_pars` to generate its master Parameters list.
    
    Parameters
    ----------
    index : `int`
      The index of the function.

    Returns
    -------
    A unique identifier string pertaining to that function, which
    can be used to generate unique parameter names.
    """
    return '__Func'+str(index)+'__'


# Licensed under a 3-clause BSD style license - see LICENSE.rst

import sys

import pytest

from . import *
from ....utils import iteritems

pytest.importorskip('sphinx')  # skips these tests if sphinx not present


class FakeEnv(object):
    """
    Mocks up a sphinx env setting construct for automodapi tests
    """
    def __init__(self, **kwargs):
        for k, v in iteritems(kwargs):
            setattr(self, k, v)


class FakeBuilder(object):
    """
    Mocks up a sphinx builder setting construct for automodapi tests
    """
    def __init__(self, **kwargs):
        self.env = FakeEnv(**kwargs)


class FakeApp(object):
    """
    Mocks up a `sphinx.application.Application` object for automodapi tests
    """
    def __init__(self, srcdir, automodapipresent=True):
        self.builder = FakeBuilder(srcdir=srcdir)
        self.info = []
        self.warnings = []
        self._extensions = []
        if automodapipresent:
            self._extensions.append('astropy_helpers.sphinx.ext.automodapi')

    def info(self, msg, loc):
        self.info.append((msg, loc))

    def warn(self, msg, loc):
        self.warnings.append((msg, loc))


ams_to_asmry_str = """
Before

.. automodsumm:: astropy_helpers.sphinx.ext.automodsumm
    :p:

And After
"""

ams_to_asmry_expected = """\
.. currentmodule:: astropy_helpers.sphinx.ext.automodsumm

.. autosummary::
    :p:

    Automoddiagram
    Automodsumm
    automodsumm_to_autosummary_lines
    generate_automodsumm_docs
    process_automodsumm_generation
    setup
"""


def test_ams_to_asmry(tmpdir):
    from ..automodsumm import automodsumm_to_autosummary_lines

    fi = tmpdir.join('automodsumm.rst')
    fi.write(ams_to_asmry_str)

    fakeapp = FakeApp(srcdir='')
    resultlines = automodsumm_to_autosummary_lines(str(fi), fakeapp)

    assert '\n'.join(resultlines) == ams_to_asmry_expected


ams_cython_str = """
Before

.. automodsumm:: _eva_.unit02
    :functions-only:
    :p:

And After
"""

ams_cython_expected = """\
.. currentmodule:: _eva_.unit02

.. autosummary::
    :p:

    pilot
"""


def test_ams_cython(tmpdir, cython_testpackage):
    from ..automodsumm import automodsumm_to_autosummary_lines

    fi = tmpdir.join('automodsumm.rst')
    fi.write(ams_cython_str)

    fakeapp = FakeApp(srcdir='')
    resultlines = automodsumm_to_autosummary_lines(str(fi), fakeapp)

    assert '\n'.join(resultlines) == ams_cython_expected
